/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENSYx527Def.h"
#include "CAENSYx527State.h"
#include "CAENSYx527Driver.h"
#include "PerfectCAENSYx527Driver.h"
#include "RealCAENSYx527Driver.h"
#include "drivers/global/DriversCommands.h"

namespace caen_syx527 {

const std::string CAENSYx527Driver::TYPE = "caen_syx527";

/*
 * Constructor
 */
CAENSYx527Driver::CAENSYx527Driver(const string& name):
		DeviceDriver(name){

		registerStates(new RealCAENSYx527Driver(this), new PerfectCAENSYx527Driver(this), new PerfectCAENSYx527Driver(this));

		linktype.init(this, SAVE, "linktype");
		address.init(this, SAVE, "address");
		modelcrate.init(this, SAVE, "modelcrate");
		nslot.init(this,NOSAVE,"nslot");
		nchannelperslot.init(this,NOSAVE,"nchannelperslot");
		modelboard.init(this,NOSAVE,"modelboard");
		descrboard.init(this,NOSAVE,"descrboard");
		serialboard.init(this,NOSAVE,"serialboard");
		firmboard.init(this,NOSAVE,"firmboard");

		ntotchannel.init(this, NOSAVE, "ntotchannel");
		islot.init(this, NOSAVE, "islot");
		ichannel.init(this, NOSAVE, "ichannel");
		namechannel.init(this, NOSAVE, "namechannel");
		i0set.init(this, NOSAVE, "i0set");
		v0set.init(this, NOSAVE, "v0set");
		imon.init(this, NOSAVE, "imon");
		vmon.init(this, NOSAVE, "vmon");
		power.init(this, NOSAVE, "power");
		statuschannel.init(this, NOSAVE, "statuschannel");
		rampup.init(this, NOSAVE, "rampup");
		rampdown.init(this, NOSAVE, "rampdown");

		// Init functions
		registerFunction(CAENSYX527_FUNCTION);
		registerRefresher(nslot,&CAENSYx527Driver::refreshNSlotProperty, this);
		registerRefresher(ntotchannel,&CAENSYx527Driver::refreshNTotalChannelProperty, this);
		registerRefresher(modelcrate,&CAENSYx527Driver::refreshModelCrateProperty, this);
		/*
			 * Register the Spy and Observer commands necessary to do the updates.
		*/
		registerSpyCommand(driver::READ_COMMAND, 1);//5 sec
		registerObserverCommand(driver::READ_COMMAND, 1);//20 msec

		sysHndl=-1;
		model=SY5527;
		address="0.0.0.0";
		locked = false;
}

/*
 * Destructor
 */
CAENSYx527Driver::~CAENSYx527Driver()
{
	// Empty
}

/*
 * execute
 */
void CAENSYx527Driver::execute(const string& aCommand)
{
	CAENSYx527State* currentState = dynamic_cast<CAENSYx527State*> (getCurrentState());

		if ((aCommand != driver::STOP_COMMAND) &&
				(aCommand != driver::STATUS_COMMAND) &&
				(aCommand != driver::READ_COMMAND)) {
			sendProgressEvent(PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER);
		}

		// Check command
		if (aCommand == driver::INIT_COMMAND) {
			currentState->init();
		}
		else if (aCommand == driver::READ_COMMAND) {
			currentState->readParamChannels();
		}
		else {
			//Error bad command
			//Todo , normaly it will be detect it device action driver
		}
}

void CAENSYx527Driver::refreshNSlotProperty(int32 value) throw (CannotSetValue) {
	nchannelperslot.resize(value);
	modelboard.resize(value);
	descrboard.resize(value);
	serialboard.resize(value);
	firmboard.resize(value);
}

void CAENSYx527Driver::refreshNTotalChannelProperty(int32 value) throw (CannotSetValue){
	islot.resize(value);
	ichannel.resize(value);
	namechannel.resize(value);
	i0set.resize(value);
	v0set.resize(value);
	imon.resize(value);
	vmon.resize(value);
	power.resize(value);
	statuschannel.resize(value);
	rampup.resize(value);
	rampdown.resize(value);
}

void CAENSYx527Driver::refreshModelCrateProperty(const std::string& value) throw (CannotSetValue) {
	if(value=="SY1527"){
		model=SY1527;
	}
	else if(value=="SY2527"){
		model=SY2527;
	}
	else if(value=="SY4527"){
		model=SY4527;
	}
	else if(value=="SY5527"){
		model=SY5527;
	}

}
/*
 * readParam
 */
void CAENSYx527Driver::readParamChannels() {

	CAENSYx527State* currentState = static_cast<CAENSYx527State*> (getCurrentState());
	currentState->readParamChannels();
}

/*
 * writeParam
 */
void CAENSYx527Driver::writeParam(int iChannel, string Param,float value) {

	CAENSYx527State* currentState = static_cast<CAENSYx527State*> (getCurrentState());
	currentState->writeParam(iChannel,Param,value);
}

/*
 * powerOn
 */
void CAENSYx527Driver::powerOn(int iChannel) {

	CAENSYx527State* currentState = static_cast<CAENSYx527State*> (getCurrentState());
	currentState->powerOn(iChannel);
}

/*
 * powerOff
 */
void CAENSYx527Driver::powerOff(int iChannel) {

	CAENSYx527State* currentState = static_cast<CAENSYx527State*> (getCurrentState());
	currentState->powerOff(iChannel);
}


}
